/*
 *  Gideon, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GIDEON_DESIGNER_H
#define GIDEON_DESIGNER_H

#include "gideon.h"
#include "session.h"
#include "widgets.h"

namespace Gideon {

typedef unsigned ExplorerTest;
const ExplorerTest etCanAdd       =   1;
const ExplorerTest etCanRemove    =   2;
const ExplorerTest etCanMoveUp    =   4;
const ExplorerTest etCanMoveDown  =   8;
const ExplorerTest etCanDefault   =  16;

class DesignerImpl : public Gtk::Alignment, public Designer, public Context {

public:
	DesignerImpl();

	void extend_ui(const Glib::ustring & ui, const Glib::RefPtr<Gtk::ActionGroup> & actions);
	void apply_settings(const Settings & settings);
	void store_settings(Settings * settings);
	bool can_save();
	bool should_save();
	void clear();
	void load(const Glib::ustring & data);
	Glib::ustring save();
	void saved();
	Glib::RefPtr<Gtk::AccelGroup> get_accel_group();
	Glib::ustring save_header();
	void set_embedded(bool embedded);

	void sweep();
	Gtk::Window * getWindow();
	Settings * getSettings() { return &settings; }

protected:
	PGtkActionGroup createActionGroup();
	PGtkUIManager createUIManager(PGtkActionGroup actionGroup);
	void showChangedWarning();

	void onActionUndo();
	void onActionRedo();
	void onActionCut();
	void onActionCopy();
	void onActionPaste();
	void onActionPasteToRoot();
	void onActionDelete();
	void onActionAbout();
	void onActionHome();
	void onActionForum();
	void onActionPreview();
	void onActionDefaultSize();
	void onActionShiftLeft();
	void onActionShiftRight();
	void onActionTranslatables();
	void onActionDefault();
	void onActionAdd();
	void onActionRemove();
	void onActionMoveUp();
	void onActionMoveDown();

	void updateActions();
	void updateSaveAction();
	void updateExplorerActions();

	void onClipboardGet(Gtk::SelectionData & data, guint);
	void onClipboardClear();
	void onClipboardReceived(const Gtk::SelectionData & data, bool root);

	void onManagerActionStarted();
	void onManagerActionCompleted();
	void onExplorerSelectionChanged();

	void onBeforeSessionsUpdate();
	void onAfterSessionsUpdate();
	void onHierarchySessionEvent(PSession);
	void onCanvasSessionEvent(PSession);
	void onPropertiesSessionEvent(PSession);
	void onSignalsSessionEvent(PSession);
	void onPackingSessionEvent(PSession);
	void onAfterLoad(PSession session);

	void setRowValues(PPolyelem elem);

	void onPaletteEvent(TypeId type, PaletteEvent event);
	void paste(const Glib::ustring & data, bool root, bool normal);
	void onPastingFinished();
	void onStatusChanged(const Glib::ustring & text);
	void onPopupMenu();

	bool canDefaultSize();
	void setDefaultSize();

	typedef std::pair<PSession, PNode> Selected;
	Selected getSelected();

	bool signalsPageCurrent();
	ExplorerTest testExplorer();

	void doDefault();
	void doAdd();
	void doRemove();
	void doMoveUp();
	void doMoveDown();

	bool canPreview();
	void doPreview();

	void onLeaks();

	int getShiftCapacity(PNode child);
	PNode findShiftNeighbour(PNode child, int index);
	int getChildIndex(PNode child);
	void setChildIndex(PNode child, int index);
	bool canShift();
	void doShift(int shift);

private:
	int savePoint;
	Settings settings;

	Pool<NodeSet> states;
	PObject savedState;
	NodeSet savedSelection;

	Glib::ustring clipboardStore;
	std::string clipboardTarget;

	PGtkUIManager uiManager;
	PGtkActionGroup actionGroup;
	PGtkActionGroup extendedActions;
	Gtk::UIManager::ui_merge_id extendedUi;

	Gtk::VBox vBox;
	Gtk::Statusbar statusbar;
	Gtk::Frame infoFrame;
	Gtk::Label infoLabel;
	Gtk::HPaned hPaned;
	Gtk::VPaned vPanedHP;
	Gtk::HPaned hPanedCE;
	Gtk::VBox explorerVBox;

	PaletteWidget paletteWidget;

	CanvasWidget canvasWidget;
	HierarchyWidget hierarchyWidget;
	ExplorerWidget explorerWidget;

	HierarchySessionSupplier hierarchySessionSupplier;
	CanvasSessionSupplier canvasSessionSupplier;
	PropertiesSessionSupplier propertiesSessionSupplier;
	SignalsSessionSupplier signalsSessionSupplier;
	PackingSessionSupplier packingSessionSupplier;

	SessionManager manager;

};

}

#endif
