/*
 *  Gideon, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GIDEON_GIDEON_IMPL_H
#define GIDEON_GIDEON_IMPL_H

#include "gideon.h"

namespace Gideon {

class Designer : public virtual Glib::ObjectBase {

public:
	static Designer * get(GideonDesigner * gd);
	static PDesigner create();

	virtual void extend_ui(const Glib::ustring & ui, const Glib::RefPtr<Gtk::ActionGroup> & actions) = 0;
	virtual void apply_settings(const Settings & settings) = 0;
	virtual void store_settings(Settings * settings) = 0;
	virtual bool can_save() = 0;
	virtual bool should_save() = 0;
	virtual void clear() = 0;
	virtual void load(const Glib::ustring & data) = 0;
	virtual Glib::ustring save() = 0;
	virtual void saved() = 0;
	virtual Glib::RefPtr<Gtk::AccelGroup> get_accel_group() = 0;

	virtual Glib::ustring save_header() = 0;
	virtual void set_embedded(bool embedded) = 0;

};

class GideonInit {

friend class Gideon::Object;
friend class Palette;

public:
	static GideonInit * get();

	static void enter(const std::string & msg);
	static void leave();

protected:
	static GideonInit * & ptr();

	GideonInit();
	~GideonInit();

	void init();

	void snapshot(const std::string & msg);
	void check();

	Palette * getPalette() { return palette; }

private:
	Gui::Enter guiEnter;
	int refcounter;
	Palette * palette;

	#ifdef GIDEON_SANDBOX
		typedef std::set<Gideon::Object *> Set;
		Set objects;
		std::list<Set> snapshots;
	#else
		int objects;
		std::list<int> snapshots;
	#endif

	std::list<std::string> labels;

	void objectCreated(Gideon::Object * object);
	void objectDeleted(Gideon::Object * object);

};

}

#endif
