/*
 *  Gideon, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "headers.h"
#include "forward.h"
#include "views2.h"

#include "palette.h"
#include "widgets.h"
#include "controller.h"

namespace Gideon {

void RegisterViews2() {

	Palette * palette= Palette::get();

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GdkGravity", 10)).
		addValue("GDK_GRAVITY_NORTH_WEST", Gdk::GRAVITY_NORTH_WEST).
		addValue("GDK_GRAVITY_NORTH", Gdk::GRAVITY_NORTH).
		addValue("GDK_GRAVITY_NORTH_EAST", Gdk::GRAVITY_NORTH_EAST).
		addValue("GDK_GRAVITY_WEST", Gdk::GRAVITY_WEST).
		addValue("GDK_GRAVITY_CENTER", Gdk::GRAVITY_CENTER).
		addValue("GDK_GRAVITY_EAST", Gdk::GRAVITY_EAST).
		addValue("GDK_GRAVITY_SOUTH_WEST", Gdk::GRAVITY_SOUTH_WEST).
		addValue("GDK_GRAVITY_SOUTH", Gdk::GRAVITY_SOUTH).
		addValue("GDK_GRAVITY_SOUTH_EAST", Gdk::GRAVITY_SOUTH_EAST).
		addValue("GDK_GRAVITY_STATIC", Gdk::GRAVITY_STATIC);

	palette->takeEntry(new EnumEntry("GtkWindowType", 2)).
		addValue("GTK_WINDOW_TOPLEVEL", Gtk::WINDOW_TOPLEVEL).
		addValue("GTK_WINDOW_POPUP", Gtk::WINDOW_POPUP);

	palette->takeEntry(new EnumEntry("GdkWindowTypeHint", 8)).
		addValue("GDK_WINDOW_TYPE_HINT_NORMAL", Gdk::WINDOW_TYPE_HINT_NORMAL).
		addValue("GDK_WINDOW_TYPE_HINT_DIALOG", Gdk::WINDOW_TYPE_HINT_DIALOG).
		addValue("GDK_WINDOW_TYPE_HINT_MENU", Gdk::WINDOW_TYPE_HINT_MENU).
		addValue("GDK_WINDOW_TYPE_HINT_TOOLBAR", Gdk::WINDOW_TYPE_HINT_TOOLBAR).
		addValue("GDK_WINDOW_TYPE_HINT_SPLASHSCREEN", Gdk::WINDOW_TYPE_HINT_SPLASHSCREEN).
		addValue("GDK_WINDOW_TYPE_HINT_UTILITY", Gdk::WINDOW_TYPE_HINT_UTILITY).
		addValue("GDK_WINDOW_TYPE_HINT_DOCK", Gdk::WINDOW_TYPE_HINT_DOCK).
		addValue("GDK_WINDOW_TYPE_HINT_DESKTOP", Gdk::WINDOW_TYPE_HINT_DESKTOP);

	palette->takeEntry(new EnumEntry("GtkWindowPosition", 5)).
		addValue("GTK_WIN_POS_NONE", Gtk::WIN_POS_NONE).
		addValue("GTK_WIN_POS_CENTER", Gtk::WIN_POS_CENTER).
		addValue("GTK_WIN_POS_MOUSE", Gtk::WIN_POS_MOUSE).
		addValue("GTK_WIN_POS_CENTER_ALWAYS", Gtk::WIN_POS_CENTER_ALWAYS).
		addValue("GTK_WIN_POS_CENTER_ON_PARENT", Gtk::WIN_POS_CENTER_ON_PARENT);

	palette->takeEntry(new EnumEntry("GtkOrientation", 2)).
		addValue("GTK_ORIENTATION_HORIZONTAL", Gtk::ORIENTATION_HORIZONTAL).
		addValue("GTK_ORIENTATION_VERTICAL", Gtk::ORIENTATION_VERTICAL);

	palette->takeEntry(new EnumEntry("GtkToolbarStyle", 4)).
		addValue("GTK_TOOLBAR_ICONS", Gtk::TOOLBAR_ICONS).
		addValue("GTK_TOOLBAR_TEXT", Gtk::TOOLBAR_TEXT).
		addValue("GTK_TOOLBAR_BOTH", Gtk::TOOLBAR_BOTH).
		addValue("GTK_TOOLBAR_BOTH_HORIZ", Gtk::TOOLBAR_BOTH_HORIZ);

	palette->takeEntry(new TWidgetEntry<GtkWindowView>("GtkWindow", GTK_TYPE_WINDOW, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new EnumEntry("GtkResponseType", 11)).
		addValue("GTK_RESPONSE_NONE", Gtk::RESPONSE_NONE).
		addValue("GTK_RESPONSE_REJECT", Gtk::RESPONSE_REJECT).
		addValue("GTK_RESPONSE_ACCEPT", Gtk::RESPONSE_ACCEPT).
		addValue("GTK_RESPONSE_DELETE_EVENT", Gtk::RESPONSE_DELETE_EVENT).
		addValue("GTK_RESPONSE_OK", Gtk::RESPONSE_OK).
		addValue("GTK_RESPONSE_CANCEL", Gtk::RESPONSE_CANCEL).
		addValue("GTK_RESPONSE_CLOSE", Gtk::RESPONSE_CLOSE).
		addValue("GTK_RESPONSE_YES", Gtk::RESPONSE_YES).
		addValue("GTK_RESPONSE_NO", Gtk::RESPONSE_NO).
		addValue("GTK_RESPONSE_APPLY", Gtk::RESPONSE_APPLY).
		addValue("GTK_RESPONSE_HELP", Gtk::RESPONSE_HELP);

	palette->takeEntry(new TObjectEntry<DialogButtonEntryView>("GideonDialogButtonEntry", G_TYPE_INVALID));

	palette->takeEntry(new TWidgetEntry<GtkDialogView>("GtkDialog", GTK_TYPE_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TWidgetEntry<GtkColorSelectionDialogView>("GtkColorSelectionDialog", GTK_TYPE_COLOR_SELECTION_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new EnumEntry("GtkFileChooserAction", 4)).
		addValue("GTK_FILE_CHOOSER_ACTION_OPEN", Gtk::FILE_CHOOSER_ACTION_OPEN).
		addValue("GTK_FILE_CHOOSER_ACTION_SAVE", Gtk::FILE_CHOOSER_ACTION_SAVE).
		addValue("GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER).
		addValue("GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER", Gtk::FILE_CHOOSER_ACTION_CREATE_FOLDER);

	palette->takeEntry(new TObjectEntry<GtkFileFilterView>("GtkFileFilter", GTK_TYPE_FILE_FILTER));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserDialogView>("GtkFileChooserDialog", GTK_TYPE_FILE_CHOOSER_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TWidgetEntry<GtkFontSelectionDialogView>("GtkFontSelectionDialog", GTK_TYPE_FONT_SELECTION_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkPackDirection", 4)).
		addValue("GTK_PACK_DIRECTION_LTR", GTK_PACK_DIRECTION_LTR).
		addValue("GTK_PACK_DIRECTION_RTL", GTK_PACK_DIRECTION_RTL).
		addValue("GTK_PACK_DIRECTION_TTB", GTK_PACK_DIRECTION_TTB).
		addValue("GTK_PACK_DIRECTION_BTT", GTK_PACK_DIRECTION_BTT);

	palette->takeEntry(new TWidgetEntry<GtkMenuBarView>("GtkMenuBar", GTK_TYPE_MENU_BAR, TAB_UI));

	palette->takeEntry(new TWidgetEntry<GtkToolbarView>("GtkToolbar", GTK_TYPE_TOOLBAR, TAB_UI));

	palette->takeEntry(new TWidgetEntry<GtkStatusbarView>("GtkStatusbar", GTK_TYPE_STATUSBAR, TAB_UI));

	palette->takeEntry(new TObjectEntry<TooltipsEntryView>("GideonTooltipsEntry", G_TYPE_INVALID));

	palette->takeEntry(new TObjectEntry<GtkTooltipsView>("GtkTooltips", GTK_TYPE_TOOLTIPS, 1)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<GtkActionGroupView>("GtkActionGroup", GTK_TYPE_ACTION_GROUP, -2)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<GtkActionView>("GtkAction", GTK_TYPE_ACTION));

	palette->takeEntry(new TObjectEntry<GtkToggleActionView>("GtkToggleAction", GTK_TYPE_TOGGLE_ACTION));

	palette->takeEntry(new TObjectEntry<GtkRadioActionView>("GtkRadioAction", GTK_TYPE_RADIO_ACTION));

	palette->takeEntry(new TObjectEntry<GtkUIManagerView>("GtkUIManager", GTK_TYPE_UI_MANAGER, -1)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<UIDefinitionView>("GideonUIDefinition", G_TYPE_INVALID, -1)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GideonUIDefinition");

	palette->takeEntry(new TObjectEntry<RadioGroupView>("GideonRadioGroup", G_TYPE_INVALID, -3)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	// -----------------------------------

	palette->takeEntry(new EnumEntry("PangoEllipsizeMode", 4)).
		addValue("PANGO_ELLIPSIZE_NONE", Pango::ELLIPSIZE_NONE).
		addValue("PANGO_ELLIPSIZE_START", Pango::ELLIPSIZE_START).
		addValue("PANGO_ELLIPSIZE_MIDDLE", Pango::ELLIPSIZE_MIDDLE).
		addValue("PANGO_ELLIPSIZE_END", Pango::ELLIPSIZE_END);

	palette->takeEntry(new EnumEntry("GtkJustification", 4)).
		addValue("GTK_JUSTIFY_LEFT", Gtk::JUSTIFY_LEFT).
		addValue("GTK_JUSTIFY_RIGHT", Gtk::JUSTIFY_RIGHT).
		addValue("GTK_JUSTIFY_CENTER", Gtk::JUSTIFY_CENTER).
		addValue("GTK_JUSTIFY_FILL", Gtk::JUSTIFY_FILL);

	palette->takeEntry(new TWidgetEntry<GtkLabelView>("GtkLabel", GTK_TYPE_LABEL, TAB_STANDARD));

	palette->takeEntry(new EnumEntry("GtkReliefStyle", 3)).
		addValue("GTK_RELIEF_NORMAL", Gtk::RELIEF_NORMAL).
		addValue("GTK_RELIEF_HALF", Gtk::RELIEF_HALF).
		addValue("GTK_RELIEF_NONE", Gtk::RELIEF_NONE);

	palette->takeEntry(new TWidgetEntry<GtkButtonView>("GtkButton", GTK_TYPE_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkToggleButtonView>("GtkToggleButton", GTK_TYPE_TOGGLE_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkCheckButtonView>("GtkCheckButton", GTK_TYPE_CHECK_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkRadioButtonView>("GtkRadioButton", GTK_TYPE_RADIO_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TObjectEntry<GtkEntryCompletionView>("GtkEntryCompletion", GTK_TYPE_ENTRY_COMPLETION));

	palette->takeEntry(new TWidgetEntry<GtkEntryView>("GtkEntry", GTK_TYPE_ENTRY, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkComboBoxView>("GtkComboBox", GTK_TYPE_COMBO_BOX, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkComboBoxEntryView>("GtkComboBoxEntry", GTK_TYPE_COMBO_BOX_ENTRY, TAB_STANDARD));

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkSpinButtonUpdatePolicy", 2)).
		addValue("GTK_UPDATE_ALWAYS", Gtk::UPDATE_ALWAYS).
		addValue("GTK_UPDATE_IF_VALID", Gtk::UPDATE_IF_VALID);

	palette->takeEntry(new TWidgetEntry<GtkSpinButtonView>("GtkSpinButton", GTK_TYPE_SPIN_BUTTON, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkUpdateType", 3)).
		addValue("GTK_UPDATE_CONTINUOUS", Gtk::UPDATE_CONTINUOUS).
		addValue("GTK_UPDATE_DISCONTINUOUS", Gtk::UPDATE_DISCONTINUOUS).
		addValue("GTK_UPDATE_DELAYED", Gtk::UPDATE_DELAYED);

	palette->takeEntry(new TWidgetEntry<GtkHScaleView>("GtkHScale", GTK_TYPE_HSCALE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkVScaleView>("GtkVScale", GTK_TYPE_VSCALE, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkProgressBarOrientation", 4)).
		addValue("GTK_PROGRESS_LEFT_TO_RIGHT", Gtk::PROGRESS_LEFT_TO_RIGHT).
		addValue("GTK_PROGRESS_RIGHT_TO_LEFT", Gtk::PROGRESS_RIGHT_TO_LEFT).
		addValue("GTK_PROGRESS_BOTTOM_TO_TOP", Gtk::PROGRESS_BOTTOM_TO_TOP).
		addValue("GTK_PROGRESS_TOP_TO_BOTTOM", Gtk::PROGRESS_TOP_TO_BOTTOM);

	palette->takeEntry(new TWidgetEntry<GtkProgressBarView>("GtkProgressBar", GTK_TYPE_PROGRESS_BAR, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkIconSize", 6)).
		addValue("GTK_ICON_SIZE_MENU", Gtk::ICON_SIZE_MENU).
		addValue("GTK_ICON_SIZE_SMALL_TOOLBAR", Gtk::ICON_SIZE_SMALL_TOOLBAR).
		addValue("GTK_ICON_SIZE_LARGE_TOOLBAR", Gtk::ICON_SIZE_LARGE_TOOLBAR).
		addValue("GTK_ICON_SIZE_BUTTON", Gtk::ICON_SIZE_BUTTON).
		addValue("GTK_ICON_SIZE_DND", Gtk::ICON_SIZE_DND).
		addValue("GTK_ICON_SIZE_DIALOG", Gtk::ICON_SIZE_DIALOG);

	palette->takeEntry(new TWidgetEntry<GtkImageView>("GtkImage", GTK_TYPE_IMAGE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkHSeparatorView>("GtkHSeparator", GTK_TYPE_HSEPARATOR, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkVSeparatorView>("GtkVSeparator", GTK_TYPE_VSEPARATOR, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkSizeGroupMode", 4)).
		addValue("GTK_SIZE_GROUP_NONE", Gtk::SIZE_GROUP_NONE).
		addValue("GTK_SIZE_GROUP_HORIZONTAL", Gtk::SIZE_GROUP_HORIZONTAL).
		addValue("GTK_SIZE_GROUP_VERTICAL", Gtk::SIZE_GROUP_VERTICAL).
		addValue("GTK_SIZE_GROUP_BOTH", Gtk::SIZE_GROUP_BOTH);

	palette->takeEntry(new TObjectEntry<GtkSizeGroupView>("GtkSizeGroup", GTK_TYPE_SIZE_GROUP, 1)).
		setPaletteTab(TAB_ADDITIONAL).
		setCanvasEditor("canvas|GlibObject");

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkWrapMode", 4)).
		addValue("GTK_WRAP_NONE", Gtk::WRAP_NONE).
		addValue("GTK_WRAP_CHAR", Gtk::WRAP_CHAR).
		addValue("GTK_WRAP_WORD", Gtk::WRAP_WORD).
		addValue("GTK_WRAP_WORD_CHAR", Gtk::WRAP_WORD_CHAR);

	palette->takeEntry(new TWidgetEntry<GtkTextViewView>("GtkTextView", GTK_TYPE_TEXT_VIEW, TAB_VIEWS));

	palette->takeEntry(new EnumEntry("GtkSelectionMode", 4)).
		addValue("GTK_SELECTION_NONE", Gtk::SELECTION_NONE).
		addValue("GTK_SELECTION_SINGLE", Gtk::SELECTION_SINGLE).
		addValue("GTK_SELECTION_BROWSE", Gtk::SELECTION_BROWSE).
		addValue("GTK_SELECTION_MULTIPLE", Gtk::SELECTION_MULTIPLE);

	palette->takeEntry(new TWidgetEntry<GtkIconViewView>("GtkIconView", GTK_TYPE_ICON_VIEW, TAB_VIEWS));

	palette->takeEntry(new TWidgetEntry<GtkCellViewView>("GtkCellView", GTK_TYPE_CELL_VIEW, TAB_VIEWS));

	palette->takeEntry(new TWidgetEntry<GtkTreeViewView>("GtkTreeView", GTK_TYPE_TREE_VIEW, TAB_VIEWS));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkColorButtonView>("GtkColorButton", GTK_TYPE_COLOR_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkColorSelectionView>("GtkColorSelection", GTK_TYPE_COLOR_SELECTION, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserButtonView>("GtkFileChooserButton", GTK_TYPE_FILE_CHOOSER_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserWidgetView>("GtkFileChooserWidget", GTK_TYPE_FILE_CHOOSER_WIDGET, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFontButtonView>("GtkFontButton", GTK_TYPE_FONT_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFontSelectionView>("GtkFontSelection", GTK_TYPE_FONT_SELECTION, TAB_SELECTORS));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkFixedView>("GtkFixed", GTK_TYPE_FIXED, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkLayoutView>("GtkLayout", GTK_TYPE_LAYOUT, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkHScrollbarView>("GtkHScrollbar", GTK_TYPE_HSCROLLBAR, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkVScrollbarView>("GtkVScrollbar", GTK_TYPE_VSCROLLBAR, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkEventBoxView>("GtkEventBox", GTK_TYPE_EVENT_BOX, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkDrawingAreaView>("GtkDrawingArea", GTK_TYPE_DRAWING_AREA, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkCalendarView>("GtkCalendar", GTK_TYPE_CALENDAR, TAB_SPECIAL));

	palette->takeEntry(new EnumEntry("GtkArrowType", 4)).
		addValue("GTK_ARROW_UP", Gtk::ARROW_UP).
		addValue("GTK_ARROW_DOWN", Gtk::ARROW_DOWN).
		addValue("GTK_ARROW_LEFT", Gtk::ARROW_LEFT).
		addValue("GTK_ARROW_RIGHT", Gtk::ARROW_RIGHT);

	palette->takeEntry(new TWidgetEntry<GtkArrowView>("GtkArrow", GTK_TYPE_ARROW, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<CustomView>("GideonCustom", GTK_TYPE_WIDGET, TAB_SPECIAL));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<DesignerView>("GideonDesigner", GTK_TYPE_WIDGET, TAB_DEBUG)).
		setFlags(efDebug);

	palette->takeEntry(new TWidgetEntry<HRefView>("GideonHRef", GTK_TYPE_WIDGET, TAB_DEBUG)).
		setFlags(efDebug);

	palette->takeEntry(new TWidgetEntry<WindowView>("GideonWindow", GTK_TYPE_WINDOW, TAB_DEBUG)).
		setFlags(efDebug | efTopMaster);

}

// GtkWindowView ---------------------------------------------------------------

GtkWindowView::GtkWindowView() {
	addAppPaintableProperty();
	addEventProperties();
	findProperty("visible")->
		setInertDefault(CAny::createBool(false)).
		setFlag(sfReadOnly);
	addInertProperty("accept-focus", prScalar, "bool", CAny::createBool(true));
	addInertProperty("decorated", prScalar, "bool", CAny::createBool(true));
	addInertProperty("default-size", prScalar, "GideonPoint", CAny::createPoint(Point(-1, -1)));
	addInertProperty("focus-on-map", prScalar, "bool", CAny::createBool(true));
	addInertProperty("gravity", prScalar, "GdkGravity", CAny::createEnum("GdkGravity", Gdk::GRAVITY_NORTH_WEST));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("modal", prScalar, "bool", CAny::createBool(false));
	addInertProperty("resizable", prScalar, "bool", CAny::createBool(true));
	addInertProperty("role", prScalar, "string", CAny::createString(""));
	addInertProperty("skip-pager-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("skip-taskbar-hint", prScalar, "bool", CAny::createBool(false));
	addProperty("title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("type", prScalar, "GtkWindowType", CAny::createEnum("GtkWindowType", Gtk::WINDOW_TOPLEVEL));
	addInertProperty("type-hint", prScalar, "GdkWindowTypeHint", CAny::createEnum("GdkWindowTypeHint", Gdk::WINDOW_TYPE_HINT_NORMAL));
	addInertProperty("window-position", prScalar, "GtkWindowPosition", CAny::createEnum("GtkWindowPosition", Gtk::WIN_POS_NONE));
	addInertProperty("accel-groups", prVector, "GtkUIManager").
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertObject)).
		setFlag(sfLinkOnly);
	addInertProperty("transient-for", prEntity, "GtkWindow").
		setFlag(sfLinkOnly | sfWeakLink);
	addInertProperty("default", prEntity, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink);
	addInertProperty("urgency-hint", prScalar, "bool", CAny::createBool(false));
}

Gtk::Widget * GtkWindowView::createDesignWindow() {
	Gtk::Widget * window= new DesignWindow();
	SetData(window, "GideonTypeHint", "GtkWindow");
	return window;
}

Any GtkWindowView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr(createDesignWindow()));
}

void GtkWindowView::initDesign() {
	GtkBinView::initDesign();
	getObject()->set_property("title", getDesignLabel());
}

// DialogButtonEntryView -------------------------------------------------------

DialogButtonEntry::DialogButtonEntry() {
	SetData(this, "GideonTypeHint", "GideonDialogButtonEntry");
	response= Gtk::RESPONSE_NONE;
	secondary= false;
}

DialogButtonEntryView::DialogButtonEntryView() {
	addProperty("widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getWidget)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setWidget));
	addProperty("response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE)).
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getResponse)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setResponse));
	addProperty("secondary", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getSecondary)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setSecondary));
}

Any DialogButtonEntryView::getWidget(Property * property) {
	return CAny::createObject(getObject()->widget);
}

void DialogButtonEntryView::setWidget(Property * property, Any value) {
	getObject()->widget= PGtkWidget::cast_dynamic(value->getObject());
}

Any DialogButtonEntryView::getResponse(Property * property) {
	return CAny::createEnum("GtkResponseType", getObject()->response);
}

void DialogButtonEntryView::setResponse(Property * property, Any value) {
	getObject()->response= (Gtk::ResponseType)value->getEnum();
}

Any DialogButtonEntryView::getSecondary(Property * property) {
	return CAny::createBool(getObject()->secondary);
}

void DialogButtonEntryView::setSecondary(Property * property, Any value) {
	getObject()->secondary= value->getBool();
}

// GtkDialogView ---------------------------------------------------------------

GtkDialogView::GtkDialogView() {
	addProperty("has-separator", prScalar, "bool", CAny::createBool(true));
	findProperty("type-hint")->
		setInertDefault(CAny::createEnum("GdkWindowTypeHint", Gdk::WINDOW_TYPE_HINT_DIALOG));
	findProperty("window-position")->
		setInertDefault(CAny::createEnum("GtkWindowPosition", Gtk::WIN_POS_CENTER_ON_PARENT));
	findProperty("type")->setFlag(sfReadOnly | sfHidden);
	findProperty("app-paintable")->setFlag(sfReadOnly | sfHidden);
	findProperty("default")->setFlag(sfReadOnly | sfHidden);
	addInertProperty("buttons", prVector, "GideonDialogButtonEntry").
		setFlag(sfOverwrite).
		setValueSet(sigc::mem_fun(*this, &GtkDialogView::setButtons)).
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GtkDialogView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkDialogView::getButtonLabel));
	addInertProperty("default-response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE));
}

BinContainer * GtkDialogView::createContainer() {
	return new DesignDialogContainer();
}

Gtk::Widget * GtkDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignDialog();
	SetData(window, "GideonTypeHint", "GtkDialog");
	return window;
}

void GtkDialogView::setButtons(Property * property, Any value) {
	property->setInert(value);
	clearButtons();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PDialogButtonEntry entry= vec[i]->getObject<DialogButtonEntry>();
		if(entry && entry->widget)
			addButton(entry);
	}
}

void GtkDialogView::clearButtons() {
	ContainerClear(getObject<DesignDialog>()->getButtons());
}

void GtkDialogView::addButton(PDialogButtonEntry entry) {
	getObject<DesignDialog>()->getButtons()->add(*GetPtr(entry->widget));
	if(entry->secondary)
		getObject<DesignDialog>()->getButtons()->set_child_secondary(*GetPtr(entry->widget));
}

std::string GtkDialogView::getButtonLabel(Property * property, PNode node) {
	PNode resp= getModel()->find(node, "response");
	if(resp) {
		std::string str= resp->getValue()->toString();
		return Glib::ustring(str.substr(13)).lowercase();
	}
	return *node->getName();
}

// GtkDialogViewFinal ----------------------------------------------------------

GtkDialogViewFinal::GtkDialogViewFinal() {
	findProperty("buttons")->setFlag(sfReadOnly | sfHidden);
	findProperty("as-container")->setFlag(sfReadOnly | sfHidden);
	findProperty("focus-chain")->setFlag(sfHidden);
	findProperty("focus-child")->setFlag(sfHidden);
	findProperty("default-response")->setFlag(sfReadOnly | sfHidden);
	findProperty("accel-groups")->setFlag(sfReadOnly | sfHidden);
	findProperty("title")->setFlag(sfAutoDefault);
}

// GtkColorSelectionDialogView -------------------------------------------------

GtkColorSelectionDialogView::GtkColorSelectionDialogView() {
	addProperty("colorsel", prEntity, "GtkColorSelection").
		setValueGet(sigc::mem_fun(*this, &GtkColorSelectionDialogView::getColorsel)).
		setFlag(sfReadOnly);
	findProperty("resizable")->
		setInertDefault(CAny::createBool(false));
}

Gtk::Widget * GtkColorSelectionDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignColorSelectionDialog();
	SetData(window, "GideonTypeHint", "GtkColorSelectionDialog");
	return window;
}

Any GtkColorSelectionDialogView::getColorsel(Property * property) {
	PGtkWidget widget= MakeRefPtr(getObject<DesignColorSelectionDialog>()->getColorsel());
	return CAny::createObject(widget);
}

// GtkFileFilterView -----------------------------------------------------------

GtkFileFilterView::GtkFileFilterView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkFileFilterView::getName)).
		setValueSet(sigc::mem_fun(*this, &GtkFileFilterView::setName));
	addInertProperty("add-pixbuf-formats", prScalar, "bool", CAny::createBool(false));
	addInertProperty("patterns", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("*.*")
		));
	addInertProperty("mime-types", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
}

Any GtkFileFilterView::createInstance() {
	Gtk::FileFilter * filter= new Gtk::FileFilter();
	filter->add_pattern("*.*");
	return CAny::createObject(getType(), NewRefPtr(filter));
}

Any GtkFileFilterView::getName(Property * property) {
	return CAny::createString(getObject<Gtk::FileFilter>()->get_name());
}

void GtkFileFilterView::setName(Property * property, Any value) {
	getObject<Gtk::FileFilter>()->set_name(*value->getString());
}

// GtkFileChooserViewBase ------------------------------------------------------

GtkFileChooserViewBase::GtkFileChooserViewBase() {
	addProperty("action", prScalar, "GtkFileChooserAction", CAny::createEnum("GtkFileChooserAction", Gtk::FILE_CHOOSER_ACTION_OPEN));
	addProperty("extra-widget", prEntity, "GtkWidget");
	addInertProperty("file-system-backend", prScalar, "string", CAny::createString(""));
	addProperty("local-only", prScalar, "bool", CAny::createBool(true));
	addProperty("preview-widget", prEntity, "GtkWidget");
	addProperty("preview-widget-active", prScalar, "bool", CAny::createBool(true));
	addProperty("select-multiple", prScalar, "bool", CAny::createBool(false));
	addProperty("show-hidden", prScalar, "bool", CAny::createBool(false));
	addProperty("use-preview-label", prScalar, "bool", CAny::createBool(true));
	addInertProperty("filter", prEntity, "GtkFileFilter").
		setValueSet(sigc::mem_fun(*this, &GtkFileChooserViewBase::setFilter));
	addInertProperty("filters", prVector, "GtkFileFilter").
		setValueSet(sigc::mem_fun(*this, &GtkFileChooserViewBase::setFilters)).
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
	addProperty("do-overwrite-confirmation", prScalar, "bool", CAny::createBool(false));
}

Glib::RefPtr<Gtk::FileChooser> GtkFileChooserViewBase::getObject() {
	return Glib::RefPtr<Gtk::FileChooser>::cast_dynamic(GlibObjectView::getObject());
}

void GtkFileChooserViewBase::setFilter(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::FileFilter> filter= value->getObject<Gtk::FileFilter>();
	if(filter) {
		clearVectorProperty("filters")->setFlag(sfReadOnly);
		clearFilters();
	} else
		findProperty("filters")->clearFlag(sfReadOnly);
	touch("filters");
	getObject()->property_filter()= GetPtr(filter);
}

void GtkFileChooserViewBase::setFilters(Property * property, Any value) {
	property->setInert(value);
	clearFilters();
	Glib::RefPtr<Gtk::FileChooser> chooser= getObject();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		Glib::RefPtr<Gtk::FileFilter> obj= vec[i]->getObject<Gtk::FileFilter>();
		if(obj)
			chooser->add_filter(*GetPtr(obj));
	}
}

void GtkFileChooserViewBase::clearFilters() {
	Glib::RefPtr<Gtk::FileChooser> chooser= getObject();
	for(;;) {
		GSList * list= gtk_file_chooser_list_filters(chooser->gobj());
		if(list) {
			GtkFileFilter * filter= GTK_FILE_FILTER(list->data);
			gtk_file_chooser_remove_filter(chooser->gobj(), filter);
		} else
			break;
	}
}

bool GtkFileChooserViewBase::isActiveWidget(Gtk::Widget * widget) {
	if(
		widget==getObject()->get_preview_widget() && 
		!getObject()->get_preview_widget_active()
	)
		return false;
	else
		return true;
}

// GtkFileChooserDialogView ----------------------------------------------------

GtkFileChooserDialogView::GtkFileChooserDialogView() {
	findProperty("buttons")->clearFlag(sfReadOnly | sfHidden);
	findProperty("default-response")->clearFlag(sfReadOnly | sfHidden);
	findProperty("has-separator")->
		setInertDefault(CAny::createBool(false));
}

Glib::RefPtr<Gtk::FileChooser> GtkFileChooserDialogView::getObject() {
	return MakeRefPtr(GlibObjectView::getObject<DesignFileChooserDialog>()->getFileChooser());
}

Gtk::Widget * GtkFileChooserDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignFileChooserDialog();
	SetData(window, "GideonTypeHint", "GtkFileChooserDialog");
	return window;
}

// GtkFontSelectionDialogView --------------------------------------------------

GtkFontSelectionDialogView::GtkFontSelectionDialogView() {
	addProperty("font-selection", prEntity, "GtkFontSelection").
		setValueGet(sigc::mem_fun(*this, &GtkFontSelectionDialogView::getFontSelection)).
		setFlag(sfReadOnly);
	findProperty("has-separator")->
		setInertDefault(CAny::createBool(false));
}

Gtk::Widget * GtkFontSelectionDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignFontSelectionDialog();
	SetData(window, "GideonTypeHint", "GtkFontSelectionDialog");
	return window;
}

Any GtkFontSelectionDialogView::getFontSelection(Property * property) {
	PGtkWidget widget= MakeRefPtr(getObject<DesignFontSelectionDialog>()->getFontSelection());
	return CAny::createObject(widget);
}

// GtkStatusbarView ------------------------------------------------------------

GtkStatusbarView::GtkStatusbarView() {
	addProperty("has-resize-grip", prScalar, "bool", CAny::createBool(true));
}

// GtkTooltipsEntryView --------------------------------------------------------

TooltipsEntry::TooltipsEntry() {
	SetData(this, "GideonTypeHint", "GideonTooltipsEntry");
}

TooltipsEntryView::TooltipsEntryView() {
	addProperty("widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &TooltipsEntryView::getWidget)).
		setValueSet(sigc::mem_fun(*this, &TooltipsEntryView::setWidget)).
		setFlag(sfLinkOnly | sfWeakLink);
	addProperty("tip-text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &TooltipsEntryView::getTipText)).
		setValueSet(sigc::mem_fun(*this, &TooltipsEntryView::setTipText));
	addProperty("tip-private", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &TooltipsEntryView::getTipPrivate)).
		setValueSet(sigc::mem_fun(*this, &TooltipsEntryView::setTipPrivate));
	addInertProperty("comment", prScalar, "string", CAny::createString(""));
}

Any TooltipsEntryView::getWidget(Property * property) {
	return CAny::createObject(getObject()->widget);
}

void TooltipsEntryView::setWidget(Property * property, Any value) {
	getObject()->widget= PGtkWidget::cast_dynamic(value->getObject());
}

Any TooltipsEntryView::getTipText(Property * property) {
	return CAny::createString(getObject()->tip_text);
}

void TooltipsEntryView::setTipText(Property * property, Any value) {
	getObject()->tip_text= *value->getString();
}

Any TooltipsEntryView::getTipPrivate(Property * property) {
	return CAny::createString(getObject()->tip_private);
}

void TooltipsEntryView::setTipPrivate(Property * property, Any value) {
	getObject()->tip_private= *value->getString();
}

// GtkTooltipsView -------------------------------------------------------------

GtkTooltipsView::GtkTooltipsView() {
	addSignalsProperty();
	addInertProperty("enabled", prScalar, "bool", CAny::createBool(true));
	addInertProperty("tips", prVector, "GideonTooltipsEntry").
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GtkTooltipsView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkTooltipsView::getTipLabel));
}

std::string GtkTooltipsView::getTipLabel(Property * property, PNode node) {
	PNode link= getModel()->find(node, "widget");
	if(link) {
		PNode target= link->getLink();
		if(target)
			return *target->getName();
	}
	return *node->getName();
}

// GtkActionGroupView ----------------------------------------------------------

GtkActionGroupView::GtkActionGroupView() {
	addSignalsProperty();
	addInertProperty("name", prScalar, "string", CAny::createString(""));
	addInertProperty("actions", prVector, "GtkAction").
		setValueSet(sigc::mem_fun(*this, &GtkActionGroupView::setActions)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject)).
		setLabelGet(sigc::mem_fun(*this, &GtkActionGroupView::getActionLabel));
	addProperty("sensitive", prScalar, "bool", CAny::createBool(true));
	addProperty("visible", prScalar, "bool", CAny::createBool(true));
}

Any GtkActionGroupView::createInstance() {
	Glib::ustring name= "temporary_name";
	return CAny::createObject(getType(), Gtk::ActionGroup::create(name));
}

void GtkActionGroupView::doneInstance() {
	clearActions();
	GlibObjectView::doneInstance();
}

void GtkActionGroupView::setActions(Property * property, Any value) {
	property->setInert(value);
	clearActions();
	PGtkActionGroup group= getObject<Gtk::ActionGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PGtkAction obj= vec[i]->getObject<Gtk::Action>();
		if(obj) {
			Any accel= getController()->findView(CAny::createObject(obj))->findProperty("accelerator")->getValue();
			if(accel->getString()->empty())
				group->add(obj);
			else
				group->add(obj, Gtk::AccelKey(*accel->getString()));
		}
	}
}

void GtkActionGroupView::clearActions() {
	PGtkActionGroup group= getObject<Gtk::ActionGroup>();
	for(;;) {
		GList * list= gtk_action_group_list_actions(group->gobj());
		if(list) {
			GtkAction * action= GTK_ACTION(list->data);
			gtk_action_group_remove_action(group->gobj(), action);
		} else
			break;
	}
}

std::string GtkActionGroupView::getActionLabel(Property * property, PNode node) {
	PEntityView view= getController()->getView(node);
	if(view)
		return "'" + *view->findProperty("name")->getValue()->getString() + "'";
	else
		return *node->getName();
}

// GtkActionView ---------------------------------------------------------------

GtkActionView::GtkActionView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string").
		setFlag(sfConstruct);
	addProperty("hide-if-empty", prScalar, "bool", CAny::createBool(true));
	addProperty("is-important", prScalar, "bool", CAny::createBool(false));
	addInertProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueSet(sigc::mem_fun(*this, &GtkActionView::setLabel));
	addProperty("sensitive", prScalar, "bool", CAny::createBool(true));
	addProperty("stock-id", prScalar, "GtkStockID", CAny::create(FindType("GtkStockID"), Gtk::StockID()));
	addProperty("tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("visible", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-horizontal", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-overflown", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-vertical", prScalar, "bool", CAny::createBool(true));
	addInertProperty("accelerator", prScalar, "string", CAny::createString(""));
}

Any GtkActionView::createInstance() {
	Glib::ustring name= getPropertyModel<Glib::ustring>("name", "");
	return CAny::createObject(getType(), Gtk::Action::create(name));
}

void GtkActionView::setLabel(Property * property, Any value) {
	property->setInert(value);
	const Glib::ustring & str= *value->getString();
	PGtkAction action= getObject<Gtk::Action>();
	if(str.empty()) {
		g_object_set(action->gobj(), "label", NULL, NULL);
		Gtk::StockID stock= action->property_stock_id();
		action->property_stock_id()= stock;
	} else {
		action->property_label()= str;
	}
}

// GtkToggleActionView ---------------------------------------------------------

GtkToggleActionView::GtkToggleActionView() {
	addProperty("draw-as-radio", prScalar, "bool", CAny::createBool(false));
	addProperty("active", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &GtkToggleActionView::getActive)).
		setValueSet(sigc::mem_fun(*this, &GtkToggleActionView::setActive));
}

Any GtkToggleActionView::createInstance() {
	Glib::ustring name= getPropertyModel<Glib::ustring>("name", "");
	return CAny::createObject(getType(), Gtk::ToggleAction::create(name));
}

Any GtkToggleActionView::getActive(Property * property) {
	return CAny::createBool(getObject<Gtk::ToggleAction>()->get_active());
}

void GtkToggleActionView::setActive(Property * property, Any value) {
	getObject<Gtk::ToggleAction>()->set_active(value->getBool());
}

// GtkRadioActionView ----------------------------------------------------------

GtkRadioActionView::GtkRadioActionView() {
	addProperty("value", prScalar, "int", CAny::createInt(0));
	findProperty("draw-as-radio")->
		setDefault(CAny::createBool(true)).
		setFlag(sfReadOnly | sfHidden);
	addInertProperty("group", prEntity, "GideonRadioGroup").
		setFlag(sfWeakLink | sfLinkOnly);
	moveBefore("group", "active");
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkRadioActionView::setActive));
}

Any GtkRadioActionView::createInstance() {
	Glib::ustring name= getPropertyModel<Glib::ustring>("name", "");
	Gtk::RadioAction::Group group;
	dummy= Gtk::RadioAction::create(group, "");
	Glib::RefPtr<Gtk::RadioAction> action= Gtk::RadioAction::create(group, name);
	action->property_draw_as_radio()= true;
	return CAny::createObject(getType(), action);
}

void GtkRadioActionView::setActive(Property * property, Any value) {
	dummy->set_active(!value->getBool());
	getObject<Gtk::RadioAction>()->set_active(value->getBool());
}

// GtkUIManagerView ------------------------------------------------------------

GtkUIManagerView::GtkUIManagerView() {
	addSignalsProperty();
	addProperty("add-tearoffs", prScalar, "bool", CAny::createBool(false));
	addInertProperty("action-groups", prVector, "GtkActionGroup").
		setFlag(sfLinkOnly).
		setValueSet(sigc::mem_fun(*this, &GtkUIManagerView::setActionGroups)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
	addInertProperty("ui-definitions", prVector, "GideonUIDefinition").
		setFlag(sfLinkOnly).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
}

Gtk::UIManager::ui_merge_id GtkUIManagerView::getMergeId(int index) {
	Check(0 <= index && index < (int)mergeIds.size());
	return mergeIds[index];
}

Any GtkUIManagerView::createInstance() {
	PGtkUIManager manager= Gtk::UIManager::create();
	return CAny::createObject(getType(), manager);
}

void GtkUIManagerView::setActionGroups(Property * property, Any value) {
	property->setInert(value);
	PGtkUIManager manager= getObject<Gtk::UIManager>();
	for(;;) {
		GList * list= gtk_ui_manager_get_action_groups(manager->gobj());
		if(list) {
			GtkActionGroup * group= GTK_ACTION_GROUP(list->data);
			gtk_ui_manager_remove_action_group(manager->gobj(), group);
		} else
			break;
	}
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PGtkActionGroup obj= vec[i]->getObject<Gtk::ActionGroup>();
		if(obj)
			manager->insert_action_group(obj);
	}
}

// UIDefinitionView ------------------------------------------------------------

UIDefinition::UIDefinition() :
	Glib::ObjectBase(typeid(UIDefinition)),
	propertyUi(*this, "ui", "<ui>\n</ui>\n")
{
	SetData(this, "GideonTypeHint", "GideonUIDefinition");
}

UIDefinitionView::UIDefinitionView() {
	addProperty("ui", prScalar, "string", CAny::createString("<ui>\n</ui>\n")).
		setFlag(sfHidden | sfNoEditing | sfXml);
}

// UIElementView ---------------------------------------------------------------

UIElementView::UIElementView() {
	//setReportDestruction(false);
	addInertProperty("element", prScalar, "string", CAny::createString("")).
		setEditor("element|string");
	addInertProperty("manager", prEntity, "GtkUIManager").
		setFlag(sfWeakLink | sfLinkOnly);
}

Any UIElementView::createInstance() {
	return CAny::createObject(getType(), createStub());
}

// GtkMenuBarView --------------------------------------------------------------

GtkMenuBarView::GtkMenuBarView() {
	addEventProperties();
	addProperty("take-focus", prScalar, "bool", CAny::createBool(true));
	addProperty("child-pack-direction", prScalar, "GtkPackDirection", CAny::createEnum("GtkPackDirection", GTK_PACK_DIRECTION_LTR));
	addProperty("pack-direction", prScalar, "GtkPackDirection", CAny::createEnum("GtkPackDirection", GTK_PACK_DIRECTION_LTR));
}

bool GtkMenuBarView::validWidget(Gtk::Widget * widget) {
	return dynamic_cast<Gtk::MenuBar *>(widget);
}

PGtkWidget GtkMenuBarView::createStub() {
	Gtk::MenuBar * menuBar= new Gtk::MenuBar();
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("File"));
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("Edit"));
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("Help"));
	return NewRefPtr(menuBar);
}

// GtkToolbarView --------------------------------------------------------------

GtkToolbarView::GtkToolbarView() {
	addProperty("orientation", prScalar, "GtkOrientation", CAny::createEnum("GtkOrientation", Gtk::ORIENTATION_HORIZONTAL));
	addProperty("show-arrow", prScalar, "bool", CAny::createBool(true));
	addInertProperty("toolbar-style-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setToolbarStyleSet));
	addInertGetProperty("toolbar-style", prScalar, "GtkToolbarStyle", CAny::createEnum("GtkToolbarStyle", Gtk::TOOLBAR_ICONS)).
		setFlag(sfOverwrite | sfReadOnly);
	addProperty("tooltips", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &GtkToolbarView::getTooltips)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setTooltips));
}

bool GtkToolbarView::validWidget(Gtk::Widget * widget) {
	return dynamic_cast<Gtk::Toolbar *>(widget);
}

PGtkWidget GtkToolbarView::createStub() {
	Gtk::Toolbar * toolbar= new Gtk::Toolbar();
	Gtk::ToolButton * b1= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::NEW));
	Gtk::ToolButton * b2= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::OPEN));
	Gtk::ToolButton * b3= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::SAVE));
	b1->show();
	b2->show();
	b3->show();
	toolbar->append(*b1);
	toolbar->append(*b2);
	toolbar->append(*b3);
	return NewRefPtr(toolbar);
}

void GtkToolbarView::setToolbarStyleSet(Property * property, Any value) {
	property->setInert(value);
	if(value->getBool()) {
		findProperty("toolbar-style")->clearFlag(sfReadOnly);
		getObject<Gtk::Toolbar>()->set_toolbar_style(Gtk::TOOLBAR_ICONS);
	} else {
		findProperty("toolbar-style")->
			setFlag(sfReadOnly).
			setInert(CAny::createEnum("GtkToolbarStyle", Gtk::TOOLBAR_ICONS));
		getObject<Gtk::Toolbar>()->unset_toolbar_style();
	}
	touch("toolbar-style");
}

Any GtkToolbarView::getTooltips(Property * property) {
	return CAny::createBool(getObject<Gtk::Toolbar>()->get_tooltips());
}

void GtkToolbarView::setTooltips(Property * property, Any value) {
	getObject<Gtk::Toolbar>()->set_tooltips(value->getBool());
}

// RadioGroupView --------------------------------------------------------------

RadioGroup::RadioGroup() {
	SetData(this, "GideonTypeHint", "GideonRadioGroup");
}

// GtkMiscView -----------------------------------------------------------------

GtkMiscView::GtkMiscView() {
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("xpad", prScalar, "int", CAny::createInt(0));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("ypad", prScalar, "int", CAny::createInt(0));
}

// GtkLabelView ----------------------------------------------------------------

GtkLabelView::GtkLabelView() {
	addProperty("angle", prScalar, "double", CAny::createDouble(0.0));
	addProperty("ellipsize", prScalar, "PangoEllipsizeMode", CAny::createEnum("PangoEllipsizeMode", Pango::ELLIPSIZE_NONE));
	addProperty("justify", prScalar, "GtkJustification", CAny::createEnum("GtkJustification", Gtk::JUSTIFY_LEFT));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("max-width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("mnemonic-widget", prEntity, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink);
	addProperty("selectable", prScalar, "bool", CAny::createBool(false));
	addProperty("single-line-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("use-markup", prScalar, "bool", CAny::createBool(false));
	addProperty("use-underline", prScalar, "bool", CAny::createBool(false));
	addProperty("wrap", prScalar, "bool", CAny::createBool(false));
}

void GtkLabelView::initDesign() {
	getObject()->set_label(getDesignLabel());
}

// GtkButtonViewBase -----------------------------------------------------------

GtkButtonViewBase::GtkButtonViewBase() {
	addReceivesDefaultProperty(true);
	addCanFocusProperty(true);
	addCanDefaultProperty(false);
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
	addProperty("relief", prScalar, "GtkReliefStyle", CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NORMAL));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
}

// GtkButtonView ---------------------------------------------------------------

GtkButtonView::GtkButtonView() {
	addInertProperty("stock-id", prScalar, "GtkStockID", CAny::create(FindType("GtkStockID"), Gtk::StockID())).
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setStockId));
	addInertGetProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("image", prEntity, "GtkWidget").
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setImage));
	addProperty("use-underline", prScalar, "bool", CAny::createBool(false));
}

void GtkButtonView::initDesign() {
	findProperty("label")->setValue(CAny::createString(getDesignLabel()));
}

void GtkButtonView::setupContainer() {
	if(isAsContainer()) {
		resetButton();
		BinRemove(GetPtr(getObject()));
		GtkBinView::setupContainer();
	} else {
		BinRemove(GetPtr(getObject()));
		resetButton();
	}
}

void GtkButtonView::onAsContainerSetup() {
	setState(isAsContainer() ? bsContainer : bsNormal);
}

void GtkButtonView::resetButton() {
	Gtk::Button * button= GetPtr(getObject());
	if(button->get_use_stock()) {
		button->set_use_stock(false);
		button->property_image()= NULL;
	} else
		setImageWidget(NULL);
	button->set_label("");
	button->set_use_underline(false);
}

void GtkButtonView::setState(ButtonState bs) {

	switch(bs) {

		case bsNormal:
			findProperty("label")->clearFlag(sfReadOnly);
			findProperty("image")->clearFlag(sfReadOnly);
			findProperty("stock-id")->clearFlag(sfReadOnly);
			findProperty("use-underline")->clearFlag(sfReadOnly);
			break;
		
		case bsStock:
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->clearFlag(sfReadOnly);
			findProperty("use-underline")->setFlag(sfReadOnly);
			break;
		
		case bsContainer:
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->setFlag(sfReadOnly).setInert(CAny::create(FindType("GtkStockID"), Gtk::StockID()));
			findProperty("use-underline")->setFlag(sfReadOnly);
			break;		
		
		default:
			Check(false);

	}

	touch("label");
	touch("image");
	touch("stock-id");
	touch("use-underline");
}

void GtkButtonView::setStockId(Property * property, Any value) {
	property->setInert(value);
	Glib::ustring str= value->get<Gtk::StockID>()->get_string();
	setState(str.empty() ? bsNormal : bsStock);
	resetButton();
	if(!str.empty()) {
		getObject()->set_label(str);
		getObject()->set_use_stock(true);
	}
}

void GtkButtonView::setImage(Property * property, Any value) {
	property->setInert(value);
	setImageWidget(GetPtr(value->getObject<Gtk::Widget>()));
}

void GtkButtonView::setImageWidget(Gtk::Widget * widget) {
	Check(getObject()->get_use_stock()==false && (!isAsContainer() || widget==NULL));
	Gtk::Widget * old= getObject()->property_image();
	Gtk::Alignment * bin= dynamic_cast<Gtk::Alignment *>(old);
	if(bin)
		BinRemove(bin);
	if(widget) {
		bin= Gtk::manage(new Gtk::Alignment());
		bin->show();
		bin->add(*widget);
		getObject()->property_image()= bin;
	} else
		getObject()->property_image()= NULL;
}

bool GtkButtonView::isActiveWidget(Gtk::Widget * widget) {
	Gtk::Widget * image= getObject()->property_image();
	Gtk::Alignment * bin= dynamic_cast<Gtk::Alignment *>(image);
	if(bin && widget == bin->get_child())
		return Gtk::Settings::get_default()->property_gtk_button_images();
	else
		return true;
}

// GtkToggleButtonView ---------------------------------------------------------

GtkToggleButtonView::GtkToggleButtonView() {
	setInteractive(true);
	addProperty("active", prScalar, "bool", CAny::createBool(false));
	addProperty("inconsistent", prScalar, "bool", CAny::createBool(false));
}

Any GtkToggleButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Gtk::ToggleButton>());
}

// GtkCheckButtonView ----------------------------------------------------------

GtkCheckButtonView::GtkCheckButtonView() {
	addReceivesDefaultProperty(false);
	addProperty("draw-indicator", prScalar, "bool", CAny::createBool(true));
}

Any GtkCheckButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Gtk::CheckButton>());
}

// GtkRadioButtonView ----------------------------------------------------------

GtkRadioButtonView::GtkRadioButtonView() {
	addInertProperty("group", prEntity, "GideonRadioGroup").
		setFlag(sfWeakLink | sfLinkOnly);
	moveBefore("group", "active");
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkRadioButtonView::setActive));
}

Any GtkRadioButtonView::createInstance() {
	Gtk::RadioButton::Group group;
	dummy= NewRefPtr(new Gtk::RadioButton(group));
	Glib::RefPtr<Gtk::RadioButton> button= NewRefPtr(new Gtk::RadioButton(group));
	return CAny::createObject(getType(), button);
}

void GtkRadioButtonView::setActive(Property * property, Any value) {
	dummy->set_active(!value->getBool());
	getObject<Gtk::RadioButton>()->set_active(value->getBool());
}

// GtkEntryCompletionView ------------------------------------------------------

GtkEntryCompletionView::GtkEntryCompletionView() {
	addSignalsProperty();
	addProperty("inline-completion", prScalar, "bool", CAny::createBool(false));
	addProperty("minimum-key-length", prScalar, "int", CAny::createInt(1));
	addProperty("popup-completion", prScalar, "bool", CAny::createBool(true));
	addInertProperty("strings", prVector, "string").
		setFlag(sfTranslatable).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("text")
		));
	addProperty("popup-set-width", prScalar, "bool", CAny::createBool(true));
	addProperty("popup-single-match", prScalar, "bool", CAny::createBool(true));
}

Any GtkEntryCompletionView::createInstance() {
	return CAny::createObject(getType(), Gtk::EntryCompletion::create());
}

// GtkEntryView ----------------------------------------------------------------

GtkEntryView::GtkEntryView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("activates-default", prScalar, "bool", CAny::createBool(false));
	addProperty("editable", prScalar, "bool", CAny::createBool(true));
	addProperty("has-frame", prScalar, "bool", CAny::createBool(true));
	addProperty("invisible-char", prScalar, "char").
		setFlag(sfAutoDefault);
	addProperty("max-length", prScalar, "int", CAny::createInt(0));
	addProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("visibility", prScalar, "bool", CAny::createBool(true));
	addProperty("width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0));
	addProperty("completion", prEntity, "GtkEntryCompletion").
		setValueGet(sigc::mem_fun(*this, &GtkEntryView::getCompletion)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setCompletion));
}

void GtkEntryView::initDesign() {
	getObject()->set_text(getDesignLabel());
}

Any GtkEntryView::getCompletion(Property * property) {
	return CAny::createObject(getObject()->get_completion());
}

void GtkEntryView::setCompletion(Property * property, Any value) {
	getObject()->set_completion(value->getObject<Gtk::EntryCompletion>());
}

// GtkComboBoxView -------------------------------------------------------------

GtkComboBoxView::GtkComboBoxView() {
	setInteractive(true);
	addProperty("add-tearoffs", prScalar, "bool", CAny::createBool(false));
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
	addInertProperty("text-mode", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxView::setTextMode));
	addInertProperty("strings", prVector, "string").
		setFlag(sfReadOnly | sfTranslatable).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxView::setStrings)).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertScalar),
			CAny::createString("text")
		));
	addProperty("active", prScalar, "int", CAny::createInt(-1));
}

Any GtkComboBoxView::createInstance() {
	Gtk::Widget * widget= new Gtk::ComboBoxText();
	SetData(widget, "GideonTypeHint", "GtkComboBox");
	return CAny::createObject(getType(), NewRefPtr(widget));
}

void GtkComboBoxView::setTextMode(Property * property, Any value) {
	property->setInert(value);
	if(value->getBool()) {
		findProperty("strings")->clearFlag(sfReadOnly);
	} else {
		clearVectorProperty("strings")->setFlag(sfReadOnly);
		clearStrings();
	}
	touch("strings");
}

void GtkComboBoxView::setStrings(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::ComboBoxText> combo= getObject<Gtk::ComboBoxText>();
	int active= combo->property_active();
	combo->clear_items();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		const Glib::ustring & str= *vec[i]->getString();
		combo->append_text(str);
	}
	combo->property_active()= active;
}

void GtkComboBoxView::clearStrings() {
	getObject<Gtk::ComboBoxText>()->clear_items();
}

// GtkComboBoxEntryView --------------------------------------------------------

GtkComboBoxEntryView::GtkComboBoxEntryView() {
	addProperty("has-frame", prScalar, "bool", CAny::createBool(true));
	addProperty("text", prScalar, "string", CAny::createString("")).
		setValueGet(sigc::mem_fun(*this, &GtkComboBoxEntryView::getText)).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxEntryView::setText));
	addProperty("entry", prEntity, "GtkEntry").
		setValueGet(sigc::mem_fun(*this, &GtkComboBoxEntryView::getEntry)).
		setViewConfigure(sigc::mem_fun(*this, &GtkComboBoxEntryView::configureEntry)).
		setFlag(sfReadOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxEntryView::setActive));
}

Any GtkComboBoxEntryView::createInstance() {
	Gtk::Widget * widget= new Gtk::ComboBoxEntryText();
	SetData(widget, "GideonTypeHint", "GtkComboBoxEntry");
	return CAny::createObject(getType(), NewRefPtr(widget));
}

Any GtkComboBoxEntryView::getEntry(Property * property) {
	return CAny::createObject(MakeRefPtr(getObject<Gtk::ComboBoxEntry>()->get_entry()));
}

Any GtkComboBoxEntryView::getText(Property * property) {
	return CAny::createString(getObject<Gtk::ComboBoxEntry>()->get_entry()->get_text());
}

void GtkComboBoxEntryView::setText(Property * property, Any value) {
	getObject<Gtk::ComboBoxEntry>()->get_entry()->set_text(*value->getString());
}

void GtkComboBoxEntryView::setActive(Property * property, Any value) {
	Gtk::ComboBoxEntry * combo= GetPtr(getObject<Gtk::ComboBoxEntry>());
	int active= combo->property_active();
	getObject<Gtk::ComboBoxEntry>()->property_active()= value->getInt();
	activeChanged(active);
}

void GtkComboBoxEntryView::configureEntry(Property * property, PEntityView view) {
	view->findProperty("has-frame")->setFlag(sfReadOnly | sfHidden);
	view->findProperty("text")->setFlag(sfReadOnly | sfHidden);
}

void GtkComboBoxEntryView::setStrings(Property * property, Any value) {
	property->setInert(value);
	Gtk::ComboBoxEntryText * combo= GetPtr(getObject<Gtk::ComboBoxEntryText>());
	int active= combo->property_active();
	combo->clear_items();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		const Glib::ustring & str= *vec[i]->getString();
		combo->append_text(str);
	}
	combo->property_active()= active;
	activeChanged(active);
}

void GtkComboBoxEntryView::clearStrings() {
	Gtk::ComboBoxEntryText * combo= GetPtr(getObject<Gtk::ComboBoxEntryText>());
	int active= combo->property_active();
	combo->clear_items();
	combo->property_active()= -1;
	activeChanged(active);
}

void GtkComboBoxEntryView::activeChanged(int old) {
	Gtk::ComboBoxEntry * combo= GetPtr(getObject<Gtk::ComboBoxEntry>());
	int actual= combo->property_active(); 
	if(actual != old) {
		if(actual > -1)
			findProperty("text")->setFlag(sfReadOnly);
		else {
			combo->get_entry()->set_text("");
			findProperty("text")->clearFlag(sfReadOnly);
		}
		touch("text");
	}
}

// GtkSpinButtonView -----------------------------------------------------------

GtkSpinButtonView::GtkSpinButtonView() {
	findProperty("text")->setFlag(sfReadOnly | sfHidden);
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly).
		setViewConfigure(sigc::mem_fun(*this, &GtkSpinButtonView::configureAdjustment));
	addProperty("climb-rate", prScalar, "double", CAny::createDouble(0));
	addProperty("digits", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("numeric", prScalar, "bool", CAny::createBool(false));
	addProperty("snap-to-ticks", prScalar, "bool", CAny::createBool(false));
	addProperty("update-policy", prScalar, "GtkSpinButtonUpdatePolicy", CAny::createEnum("GtkSpinButtonUpdatePolicy", Gtk::UPDATE_ALWAYS));
	addProperty("value", prScalar, "double", CAny::createDouble(0));
	addProperty("wrap", prScalar, "bool", CAny::createBool(false));
}

Any GtkSpinButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Gtk::SpinButton>());
}

void GtkSpinButtonView::initDesign() {
}

void GtkSpinButtonView::configureAdjustment(Property * property, PEntityView view) {
	view->findProperty("value")->setFlag(sfReadOnly);
}

// GtkRangeView ----------------------------------------------------------------

GtkRangeView::GtkRangeView() {
	setInteractive(true);
	addProperty("inverted", prScalar, "bool", CAny::createBool(false));
	addProperty("update-policy", prScalar, "GtkUpdateType", CAny::createEnum("GtkUpdateType", Gtk::UPDATE_CONTINUOUS));
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly);
}

// GtkScaleView ----------------------------------------------------------------

GtkScaleView::GtkScaleView() {
	addProperty("digits", prScalar, "int", CAny::createInt(1));
	addProperty("draw-value", prScalar, "bool", CAny::createBool(true));
	addProperty("value-pos", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_TOP));
	addCanFocusProperty(true);
}

// GtkProgressBarView ----------------------------------------------------------

GtkProgressBarView::GtkProgressBarView() {
	addEventProperties();
	addProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("fraction", prScalar, "double", CAny::createDouble(0));
	addProperty("pulse-step", prScalar, "double", CAny::createDouble(0.1));
	addProperty("orientation", prScalar, "GtkProgressBarOrientation", CAny::createEnum("GtkProgressBarOrientation", Gtk::PROGRESS_LEFT_TO_RIGHT));
	addProperty("ellipsize", prScalar, "PangoEllipsizeMode", CAny::createEnum("PangoEllipsizeMode", Pango::ELLIPSIZE_NONE));
}

// GtkImageView ----------------------------------------------------------------

GtkImageView::GtkImageView() {
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addProperty("icon-size", prScalar, "GtkIconSize", CAny::createEnum("GtkIconSize", Gtk::ICON_SIZE_BUTTON));
	addProperty("pixel-size", prScalar, "int", CAny::createInt(-1));
}

void GtkImageView::initInstance() {
	GtkMiscView::initInstance();
	getObject()->set_from_icon_name("", Gtk::ICON_SIZE_BUTTON);
}

// GtkSizeGroupView ------------------------------------------------------------

GtkSizeGroupView::GtkSizeGroupView() {
	addSignalsProperty();
	addProperty("mode", prScalar, "GtkSizeGroupMode", CAny::createEnum("GtkSizeGroupMode", Gtk::SIZE_GROUP_HORIZONTAL));
	addProperty("ignore-hidden", prScalar, "bool", CAny::createBool(false));
	addInertProperty("widgets", prVector, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink).
		setValueSet(sigc::mem_fun(*this, &GtkSizeGroupView::setWidgets)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
}

Any GtkSizeGroupView::createInstance() {
	return CAny::createObject(getType(), Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL));
}

void GtkSizeGroupView::doneInstance() {
	clearWidgets();
	GlibObjectView::doneInstance();
}

void GtkSizeGroupView::clearWidgets() {
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	for(int i=0; i<(int)widgets.size(); ++i)
		sizeGroup->remove_widget(*GetPtr(widgets[i]));
	widgets.clear();
}

void GtkSizeGroupView::setWidgets(Property * property, Any value) {
	property->setInert(value);
	clearWidgets();
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PGtkWidget obj= vec[i]->getObject<Gtk::Widget>();
		if(obj) {
			sizeGroup->add_widget(*GetPtr(obj));
			widgets.push_back(obj);
		}
	}
}

// GtkTextViewView -------------------------------------------------------------

GtkTextViewView::GtkTextViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("accepts-tab", prScalar, "bool", CAny::createBool(true));
	addProperty("cursor-visible", prScalar, "bool", CAny::createBool(true));
	addProperty("editable", prScalar, "bool", CAny::createBool(true));
	addProperty("indent", prScalar, "int", CAny::createInt(0));
	addProperty("justification", prScalar, "GtkJustification", CAny::createEnum("GtkJustification", Gtk::JUSTIFY_LEFT));
	addProperty("left-margin", prScalar, "int", CAny::createInt(0));
	addProperty("overwrite", prScalar, "bool", CAny::createBool(false));
	addProperty("pixels-above-lines", prScalar, "int", CAny::createInt(0));
	addProperty("pixels-below-lines", prScalar, "int", CAny::createInt(0));
	addProperty("pixels-inside-wrap", prScalar, "int", CAny::createInt(0));
	addProperty("right-margin", prScalar, "int", CAny::createInt(0));
	addProperty("wrap-mode", prScalar, "GtkWrapMode", CAny::createEnum("GtkWrapMode", Gtk::WRAP_NONE));
	addInertProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkTextViewView::getText)).
		setValueSet(sigc::mem_fun(*this, &GtkTextViewView::setText));
}

void GtkTextViewView::initDesign() {
	getObject()->get_buffer()->set_text(getDesignLabel());
}

Any GtkTextViewView::getText(Property * property) {
	return CAny::createString(getObject()->get_buffer()->get_text());
}

void GtkTextViewView::setText(Property * property, Any value) {
	getObject()->get_buffer()->set_text(*value->getString());
}

// GtkIconViewView -------------------------------------------------------------

GtkIconViewView::GtkIconViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("column-spacing", prScalar, "int", CAny::createInt(6));
	addProperty("columns", prScalar, "int", CAny::createInt(-1));
	addProperty("item-width", prScalar, "int", CAny::createInt(-1));
	addProperty("margin", prScalar, "int", CAny::createInt(6));
	addProperty("orientation", prScalar, "GtkOrientation", CAny::createEnum("GtkOrientation", Gtk::ORIENTATION_VERTICAL));
	addProperty("row-spacing", prScalar, "int", CAny::createInt(6));
	addProperty("selection-mode", prScalar, "GtkSelectionMode", CAny::createEnum("GtkSelectionMode", Gtk::SELECTION_SINGLE));
	addProperty("spacing", prScalar, "int", CAny::createInt(0));
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkIconViewView::setSampleData));
	addProperty("reorderable", prScalar, "bool", CAny::createBool(false));
}

void GtkIconViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkIconViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::IconView> iconView= getObject();
	if(value->getBool()) {
		PGdkPixbuf pixbuf= Gtk::IconTheme::get_default()->load_icon("gtk-dialog-warning", 32, Gtk::IconLookupFlags());
		Gtk::TreeModel::ColumnRecord columns;
		Gtk::TreeModelColumn<Glib::ustring> textColumn;
		Gtk::TreeModelColumn<PGdkPixbuf> pixbufColumn;
		columns.add(textColumn);
		columns.add(pixbufColumn);
		PGtkListStore model= Gtk::ListStore::create(columns);
		for(int i=0; i<9; ++i) {
			Gtk::TreeRow row= *model->append();
			row[textColumn]= "item" + ToString(i);
			row[pixbufColumn]= pixbuf;
		}
		iconView->set_model(model);
		iconView->set_text_column(textColumn);
		iconView->set_pixbuf_column(pixbufColumn);
	} else {
		iconView->set_model(PGtkListStore());
		iconView->queue_draw();
	}
}

// GtkCellViewView -------------------------------------------------------------

GtkCellViewView::GtkCellViewView() {
	addProperty("background-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkCellViewView::setBackgroundSet));
	addProperty("background-gdk", prScalar, "GdkColor", CAny::create("GdkColor", Gdk::Color())).
		setFlag(sfReadOnly);
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkCellViewView::setSampleData));
}

void GtkCellViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkCellViewView::setBackgroundSet(Property * property, Any value) {
	bool set= value->getBool();
	getObject()->set_background_color(Gdk::Color());
	getObject()->set_property("background-set", set);
	if(set)
		findProperty("background-gdk")->clearFlag(sfReadOnly);
	else
		findProperty("background-gdk")->setFlag(sfReadOnly);
	touch("background-gdk");
}

void GtkCellViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::CellView> cellView= getObject();
	Gtk::TreeModel::ColumnRecord columns;
	Gtk::TreeModelColumn<Glib::ustring> column0;
	Gtk::TreeModelColumn<Glib::ustring> column1;
	Gtk::TreeModelColumn<Glib::ustring> column2;
	columns.add(column0);
	columns.add(column1);
	columns.add(column2);
	PGtkListStore model= Gtk::ListStore::create(columns);
	cellView->clear();
	cellView->set_model(model);
	if(value->getBool()) {
		Gtk::TreeRow row= *model->append();
		row[column0]= "item0";
		row[column1]= "item1";
		row[column2]= "item2";
		cellView->pack_start(column0);
		cellView->pack_start(column1);
		cellView->pack_start(column2);
		cellView->set_displayed_row(model->get_path(row));
	} else {
		cellView->queue_draw();
	}
}

// GtkTreeViewView -------------------------------------------------------------

GtkTreeViewView::GtkTreeViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("enable-search", prScalar, "bool", CAny::createBool(true));
	addInertProperty("fixed-height-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("headers-visible", prScalar, "bool", CAny::createBool(true));
	addProperty("hover-expand", prScalar, "bool", CAny::createBool(false));
	addProperty("hover-selection", prScalar, "bool", CAny::createBool(false));
	addProperty("reorderable", prScalar, "bool", CAny::createBool(false));
	addProperty("rules-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("selection-mode", prScalar, "GtkSelectionMode", CAny::createEnum("GtkSelectionMode", Gtk::SELECTION_SINGLE)).
		setValueGet(sigc::mem_fun(*this, &GtkTreeViewView::getSelectionMode)).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSelectionMode));
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSampleData));
}

void GtkTreeViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkTreeViewView::doneInstance() {
	getObject()->remove_all_columns();
	getObject()->set_model(PGtkTreeModel());
	GtkContainerViewBase::doneInstance();
}

Any GtkTreeViewView::getSelectionMode(Property * property) {
	return CAny::createEnum("GtkSelectionMode", getObject()->get_selection()->get_mode());
}

void GtkTreeViewView::setSelectionMode(Property * property, Any value) {
	getObject()->get_selection()->set_mode((Gtk::SelectionMode)value->getEnum());
}

void GtkTreeViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::TreeView> treeView= getObject();
	treeView->remove_all_columns();
	if(value->getBool()) {
		Gtk::TreeModel::ColumnRecord columns;
		Gtk::TreeModelColumn<Glib::ustring> column0;
		Gtk::TreeModelColumn<Glib::ustring> column1;
		Gtk::TreeModelColumn<Glib::ustring> column2;
		columns.add(column0);
		columns.add(column1);
		columns.add(column2);
		PGtkListStore model= Gtk::ListStore::create(columns);
		for(int i=0; i<3; ++i) {
			Gtk::TreeRow row= *model->append();
			row[column0]= "item" + ToString(i) + "0";
			row[column1]= "item" + ToString(i) + "1";
			row[column2]= "item" + ToString(i) + "2";
		}
		treeView->set_model(model);
		treeView->append_column("Column0", column0);
		treeView->append_column("Column1", column1);
		treeView->append_column("Column2", column2);
	} else {
		treeView->set_model(PGtkTreeModel());
	}
}

// GtkColorButtonView ----------------------------------------------------------

GtkColorButtonView::GtkColorButtonView() {
	addProperty("alpha", prScalar, "unsigned", CAny::createUnsigned(65535));
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("use-alpha", prScalar, "bool", CAny::createBool(false));
	addProperty("color", prScalar, "GdkColor", CAny::create("GdkColor", Gdk::Color()));
}

// GtkColorSelectionView -------------------------------------------------------

GtkColorSelectionView::GtkColorSelectionView() {
	addProperty("has-opacity-control", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkColorSelectionView::setHasOpacityControl));
	addProperty("has-palette", prScalar, "bool", CAny::createBool(false));
	addProperty("current-alpha", prScalar, "unsigned", CAny::createUnsigned(65535)).
		setFlag(sfReadOnly);
	addProperty("current-color", prScalar, "GdkColor", CAny::create("GdkColor", Gdk::Color()));
}

void GtkColorSelectionView::setHasOpacityControl(Property * property, Any value) {
	bool has= value->getBool();
	getObject()->set_has_opacity_control(has);
	getObject()->set_current_alpha(65535);
	if(has)
		findProperty("current-alpha")->clearFlag(sfReadOnly);
	else
		findProperty("current-alpha")->setFlag(sfReadOnly);
	touch("current-alpha");
}

// GtkFileChooserButtonView ----------------------------------------------------

GtkFileChooserButtonView::GtkFileChooserButtonView() {
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("width-chars", prScalar, "int", CAny::createInt(-1));
}

Any GtkFileChooserButtonView::createInstance() {
	Glib::ustring backend= getPropertyModel<Glib::ustring>("file-system-backend", "");
	Gtk::FileChooserButton * widget;
	if(backend.empty())
		widget= new Gtk::FileChooserButton("", Gtk::FILE_CHOOSER_ACTION_OPEN);
	else
		widget= new Gtk::FileChooserButton("", Gtk::FILE_CHOOSER_ACTION_OPEN, backend);
	GObjectClass * oclass= G_OBJECT_GET_CLASS(widget->gobj());
	GParamSpec * property= g_object_class_find_property(oclass, "title");
	Glib::ustring title= G_PARAM_SPEC_STRING(property)->default_value;
	widget->set_title(title);
	return CAny::createObject(getType(), NewRefPtr(widget));
}

bool GtkFileChooserButtonView::isActiveWidget(Gtk::Widget * widget) {
	return false;
}

// GtkFileChooserWidgetView ----------------------------------------------------

Any GtkFileChooserWidgetView::createInstance() {
	Glib::ustring backend= getPropertyModel<Glib::ustring>("file-system-backend", "");
	if(backend.empty())
		return CAny::createObject(getType(), NewRefPtr(new Gtk::FileChooserWidget(Gtk::FILE_CHOOSER_ACTION_OPEN)));
	else
		return CAny::createObject(getType(), NewRefPtr(new Gtk::FileChooserWidget(Gtk::FILE_CHOOSER_ACTION_OPEN, backend)));
}

// GtkFontButtonView -----------------------------------------------------------

GtkFontButtonView::GtkFontButtonView() {
	addProperty("font-name", prScalar, "string").
		setFlag(sfAutoDefault);
	addProperty("show-size", prScalar, "bool", CAny::createBool(true));
	addProperty("show-style", prScalar, "bool", CAny::createBool(true));
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("use-font", prScalar, "bool", CAny::createBool(false));
	addProperty("use-size", prScalar, "bool", CAny::createBool(false));
}

// GtkFixedView ----------------------------------------------------------------

GtkFixedView::GtkFixedView() {
	addInertProperty("has-window", prScalar, "bool", CAny::createBool(false));
}

void GtkFixedView::initInstance() {
	XYContainerView::initInstance();
	getObject()->set_has_window(true);
}

void GtkFixedView::setupContainer() {
	getContainer()->set(Children());
}

void GtkFixedView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value));
}

// GtkLayoutView ---------------------------------------------------------------

GtkLayoutView::GtkLayoutView() {
	addProperty("size", prScalar, "GideonPoint", CAny::createPoint(Point(100, 100))).
		setValueGet(sigc::mem_fun(*this, &GtkLayoutView::getSize)).
		setValueSet(sigc::mem_fun(*this, &GtkLayoutView::setSize));
}

PGdkWindow GtkLayoutView::getExposeWindow() {
	return getObject()->get_bin_window();
}

void GtkLayoutView::setupContainer() {
	getContainer()->set(Children());
}

void GtkLayoutView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value));
}

Any GtkLayoutView::getSize(Property * property) {
	guint width, height;
	getObject()->get_size(width, height);
	return CAny::createPoint(Point(width, height));
}

void GtkLayoutView::setSize(Property * property, Any value) {
	const Point & point= *value->getPoint();
	getObject()->set_size(point.x, point.y);
}

// GtkScrollbarView ------------------------------------------------------------

GtkScrollbarView::GtkScrollbarView() {
	addCanFocusProperty(false);
}

// GtkEventBoxView -------------------------------------------------------------

GtkEventBoxView::GtkEventBoxView() {
	addAppPaintableProperty();
	addEventProperties();
	addInertProperty("above-child", prScalar, "bool", CAny::createBool(false));
	addInertProperty("visible-window", prScalar, "bool", CAny::createBool(true));
}

// GtkDrawingAreaView ----------------------------------------------------------

GtkDrawingAreaView::GtkDrawingAreaView() {
	addEventProperties();
}

// GtkCalendarView -------------------------------------------------------------

GtkCalendarView::GtkCalendarView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("no-month-change", prScalar, "bool", CAny::createBool(false));
	addProperty("show-day-names", prScalar, "bool", CAny::createBool(true));
	addProperty("show-heading", prScalar, "bool", CAny::createBool(true));
	addProperty("show-week-numbers", prScalar, "bool", CAny::createBool(false));
}

// GtkArrowView ----------------------------------------------------------------

GtkArrowView::GtkArrowView() {
	addProperty("arrow-type", prScalar, "GtkArrowType", CAny::createEnum("GtkArrowType", Gtk::ARROW_RIGHT));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_OUT));
}

Any GtkArrowView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr(new Gtk::Arrow(Gtk::ARROW_RIGHT, Gtk::SHADOW_OUT)));
}

// CustomView ------------------------------------------------------------------

CustomView::CustomView() {
	addInertProperty("type", prScalar, "string", CAny::createString(""));
	addInertProperty("meta", prScalar, "string", CAny::createString(""));
	addInertProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
}

Any CustomView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Custom>());
}

// DesignerView ----------------------------------------------------------------

Any DesignerView::createInstance() {
	PDesigner designer= Designer::create();
	Settings settings;
	settings["show_dw"]= "true";
	designer->apply_settings(settings);
	return CAny::createObject(getType(), PGlibObject::cast_dynamic(designer));
}

// HRefView --------------------------------------------------------------------

HRefView::HRefView() {
	addReceivesDefaultProperty(false);
	findProperty("relief")->setDefault(CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NONE));
	addProperty("text", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("url", prScalar, "string").
		setFlag(sfAutoDefault);
}

Any HRefView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<HRef>());
}

// WindowView ------------------------------------------------------------------

Gtk::Widget * WindowView::createDesignWindow() {
	Gtk::Widget * window= new DesignWindow();
	SetData(window, "GideonTypeHint", "GideonWindow");
	return window;
}

}
